﻿using System;
using System.Data;
using MySqlConnector;
using Stimulsoft.Base;
using Stimulsoft.Base.Localization;
using Stimulsoft.Report;
using Stimulsoft.Report.Dictionary;

namespace Reports.Stimulsoft_Data_Adapter
{
	public class MySqlConnectorAdapterService : StiSqlAdapterService
	{
		#region Properties
		public override string ServiceName => StiLocalization.Get("Adapters", "AdapterMySQLConnection");
		#endregion

		#region Methods
		/// <summary>
		/// Returns new data connector for this type of the database.
		/// </summary>
		/// <returns>Created connector.</returns>
		public override StiSqlDataConnector CreateConnector(string connectionString = "")
		{
			return StiMySqlConnector.Get(connectionString);
		}

		public override StiDataColumnsCollection GetColumnsFromData(StiData data, StiDataSource dataSource, CommandBehavior commandBehavior)
		{
			var dataColumns = new StiDataColumnsCollection();
			var sqlSource = dataSource as StiSqlSource;

			try
			{
				if (sqlSource.SqlCommand != null && sqlSource.SqlCommand.Length > 0 && data.Data is MySqlConnection)
				{
					var connection = data.Data as MySqlConnection;
					OpenConnection(connection, data, dataSource.Dictionary);

					using (var dataAdapter = new MySqlDataAdapter(sqlSource.SqlCommand, connection))
					using (DataTable dataTable = new DataTable())
					{
						dataTable.TableName = sqlSource.Name;

						dataAdapter.SelectCommand.CommandTimeout = sqlSource.CommandTimeout;
						dataAdapter.SelectCommand.Prepare();
						dataAdapter.Fill(dataTable);

						foreach (DataColumn column in dataTable.Columns)
						{
							dataColumns.Add(new StiDataColumn(column.ColumnName, column.Caption, column.DataType));
						}
					}

					CloseConnection(data, connection);
				}
			}
			catch (Exception e)
			{
				StiLogService.Write(this.GetType(), e);
				if (!StiOptions.Engine.HideExceptions) throw;
			}

			return dataColumns;
		}

		public override void SetDataSourceNames(StiData data, StiDataSource dataSource)
		{
			base.SetDataSourceNames(data, dataSource);

			dataSource.Name = "E2DB";
			dataSource.Alias = "E2DB";
		}

		public override Type GetDataSourceType()
		{
			return typeof(MySqlConnectorSource);
		}

		public override Type[] GetDataTypes()
		{
			return new Type[] { typeof(MySqlConnection) };
		}

		public override void ConnectDataSourceToData(StiDictionary dictionary, StiDataSource dataSource, bool loadData)
		{
			dataSource.Disconnect();

			if (!loadData)
			{
				dataSource.DataTable = new DataTable();
				return;
			}

			StiSqlSource sqlSource = dataSource as StiSqlSource;

			foreach (StiData data in dataSource.Dictionary.DataStore)
			{
				if (data.Name != sqlSource.NameInSource) continue;

				try
				{
					if (!(data.Data is MySqlConnection)) continue;

					var connection = data.ViewData as MySqlConnection;
					OpenConnection(connection, data, dataSource.Dictionary);

					sqlSource.DataAdapter = new MySqlDataAdapter(sqlSource.SqlCommand, connection);

					foreach (StiDataParameter parameter in sqlSource.Parameters)
					{
						((MySqlDataAdapter)sqlSource.DataAdapter).SelectCommand.Parameters.Add(
							parameter.Name, (MySqlDbType)parameter.Type, parameter.Size);
					}

					var dataTable = new DataTable();
					dataTable.TableName = sqlSource.Name;
					dataSource.DataTable = dataTable;

					sqlSource.DataAdapter.SelectCommand.CommandTimeout = sqlSource.CommandTimeout;
					if (loadData && sqlSource.Parameters.Count > 0)
					{
						sqlSource.DataAdapter.SelectCommand.Prepare();
						sqlSource.UpdateParameters();
					}
					else
					{
						if (loadData)
						{
							((MySqlDataAdapter)sqlSource.DataAdapter).Fill(dataTable);
							sqlSource.CheckColumnsIndexs();
						}
						else
							((MySqlDataAdapter)sqlSource.DataAdapter).FillSchema(dataTable, SchemaType.Source);
					}

					break;
				}
				catch (Exception e)
				{
					StiLogService.Write(this.GetType(), e);
					if (!StiOptions.Engine.HideExceptions) throw e;
				}
			}
		}

		public override void CreateConnectionInDataStore(StiDictionary dictionary, StiSqlDatabase database)
		{
			try
			{
				#region Remove all old data from datastore
				int index = 0;
				foreach (StiData data in dictionary.DataStore)
				{
					if (data.Name == database.Name)
					{
						dictionary.DataStore.RemoveAt(index);
						break;
					}
					index++;
				}
				#endregion

				var sqlConnection = new MySqlConnection(database.ConnectionString);
				var data2 = new StiData(database.Name, sqlConnection);
				data2.IsReportData = true;
				dictionary.DataStore.Add(data2);
			}
			catch (Exception e)
			{
				StiLogService.Write(this.GetType(), e);
				if (!StiOptions.Engine.HideExceptions) throw e;
			}
		}

		public override string TestConnection(string connectionString)
		{
			try
			{
				using (var sqlConnection = new MySqlConnection(connectionString))
				{
					sqlConnection.Open();
					sqlConnection.Close();

					return StiLocalization.Get("DesignerFx", "ConnectionSuccessfull");
				}
			}
			catch (Exception e)
			{
				return $"{StiLocalization.Get("DesignerFx", "ConnectionError")}: {e.Message}";
			}
		}
		#endregion
	}
}
